﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Interop;

namespace TestProject
{
    /// <summary>
    /// 화면 관리자
    /// </summary>
    public class ScreenManager
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 화면
        /// </summary>
        private readonly Screen screen;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 주 화면 관리자 - Primary

        /// <summary>
        /// 주 화면 관리자
        /// </summary>
        public static ScreenManager Primary
        {
            get
            {
                return new ScreenManager(Screen.PrimaryScreen);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 데스크톱 영역 - DesktopArea

        /// <summary>
        /// 데스크톱 영역
        /// </summary>
        public Rect DesktopArea
        {
            get
            {
                return GetRect(this.screen.Bounds);
            }
        }

        #endregion
        #region 작업 영역 - WorkingArea

        /// <summary>
        /// 작업 영역
        /// </summary>
        public Rect WorkingArea
        {
            get
            {
                return GetRect(this.screen.WorkingArea);
            }
        }

        #endregion
        #region 주 화면 여부 - IsPrimary

        /// <summary>
        /// 주 화면 여부
        /// </summary>
        public bool IsPrimary
        {
            get
            {
                return this.screen.Primary;
            }
        }

        #endregion
        #region 장치명 - DeviceName

        /// <summary>
        /// 장치명
        /// </summary>
        public string DeviceName
        {
            get
            {
                return this.screen.DeviceName;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 모든 화면 관리자들 구하기 - GetAllScreenManagers()

        /// <summary>
        /// 모든 화면 관리자들 구하기
        /// </summary>
        /// <returns>모든 화면 관리자들</returns>
        public static IEnumerable<ScreenManager> GetAllScreenManagers()
        {
            foreach(Screen screen in Screen.AllScreens)
            {
                yield return new ScreenManager(screen);
            }
        }

        #endregion
        #region 화면 관리자 구하기 - GetScreenManager(window)

        /// <summary>
        /// 화면 관리자 구하기
        /// </summary>
        /// <param name="window">윈도우</param>
        /// <returns>화면 관리자</returns>
        public static ScreenManager GetScreenManager(Window window)
        {
            WindowInteropHelper windowInteropHelper = new WindowInteropHelper(window);

            Screen screen = Screen.FromHandle(windowInteropHelper.Handle);

            ScreenManager screenManager = new ScreenManager(screen);

            return screenManager;
        }

        #endregion
        #region 화면 관리자 구하기 - GetScreenManager(sourcePoint)

        /// <summary>
        /// 화면 관리자 구하기
        /// </summary>
        /// <param name="sourcePoint">소스 포인트</param>
        /// <returns>화면 관리자</returns>
        public static ScreenManager GetScreenManager(System.Windows.Point sourcePoint)
        {
            int sourceX = (int)Math.Round(sourcePoint.X);
            int sourceY = (int)Math.Round(sourcePoint.Y);
            
            System.Drawing.Point targetPoint = new System.Drawing.Point(sourceX, sourceY);

            Screen screen = Screen.FromPoint(targetPoint);

            ScreenManager screenManager = new ScreenManager(screen);

            return screenManager;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Internal

        #region 생성자 - ScreenManager(screen)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="screen">화면</param>
        internal ScreenManager(Screen screen)
        {
            this.screen = screen;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 윈도우 시작 위치 설정하기 - SetWindowStartupPosition(window)

        /// <summary>
        /// 윈도우 시작 위치 설정하기
        /// </summary>
        /// <param name="window">윈도우</param>
        public static void SetWindowStartupPosition(Window window)
        {
            if(window.WindowStartupLocation == WindowStartupLocation.Manual)
            {
                return;
            }

            switch(window.WindowStartupLocation)
            {
                case WindowStartupLocation.CenterOwner :

                    Window ownerWindow = window.Owner;

                    if(ownerWindow == null)
                    {
                        return;
                    }

                    SetWindowCenterLocation(window, new Rect(ownerWindow.Left, ownerWindow.Top, ownerWindow.Width, ownerWindow.Height));

                    break;

                case WindowStartupLocation.CenterScreen :

                    ScreenManager screenManager = ScreenManager.GetScreenManager(window);

                    SetWindowCenterLocation(window, screenManager.WorkingArea);

                    break;
            }

            window.Visibility = Visibility.Visible;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region WPF 사각형 구하기 - GetRect(rectangle)

        /// <summary>
        /// WPF 사각형 구하기
        /// </summary>
        /// <param name="rectangle">WinForm 사각형</param>
        /// <returns>WPF 사각형</returns>
        private Rect GetRect(Rectangle rectangle)
        {
            return new Rect
            {
                X      = rectangle.X,
                Y      = rectangle.Y,
                Width  = rectangle.Width,
                Height = rectangle.Height
            };
        }

        #endregion
        #region 윈도우 중앙 위치 설정하기 - SetWindowCenterLocation(window, sourceRect)

        /// <summary>
        /// 윈도우 중앙 위치 설정하기
        /// </summary>
        /// <param name="window">윈도우</param>
        /// <param name="sourceRect">소스 사각형</param>
        private static void SetWindowCenterLocation(Window window, Rect sourceRect)
        {
            var centerLeft = sourceRect.X + (sourceRect.Width  / 2);
            var centerTop  = sourceRect.Y + (sourceRect.Height / 2);

            window.Left = centerLeft - (window.Width  / 2);
            window.Top  = centerTop  - (window.Height / 2);
        }

        #endregion
    }
}